module Parsec
  def SourceName(value)
    value.to_s
  end

  def Line(value)
    value.to_i
  end

  def Column(value)
    value.to_i
  end

  def SourcePos(source_name, line = nil, column = nil)
    source_name, line, column = source_name, line, column
    Object.new.tap do |object|
      object.instance_variable_set "@data", [source_name, line, column]

      def object.[](index)
        @data[index]
      end

      def object.to_s
        if self[0].nil?
          show_line_column
        else
          "\"#{self[0]}\" " + show_line_column
        end
      end

      def object.inspect
        to_s
      end

      def object.show_line_column
        "(line #{self[1]}, column #{self[2]})"
      end
    end
  end

  def newPos(name, line, column)
    SourcePos(name, line, column)
  end

  def initialPos(name)
    SourcePos(name, 1, 1)
  end

  def sourceName(source_pos)
    source_pos[0]
  end

  def sourceLine(source_pos)
    source_pos[1]
  end

  def sourceColumn(source_pos)
    source_pos[2]
  end

  def incSourceLine(source_pos, n)
    SourcePos(source_pos[0], source_pos[1] + n, source_pos[2])
  end

  def incSourceColumn(source_pos, n)
    SourcePos(source_pos[0], source_pos[1], source_pos[2] + n)
  end

  def setSourceName(source_pos, n)
    SourcePos(n, source_pos[1], source_pos[2])
  end

  def setSourceLine(source_pos, n)
    SourcePos(source_pos[0], n, source_pos[2])
  end

  def setSourceColumn(source_pos, n)
    SourcePos(source_pos[0], source_pos[1], n)
  end

  def updatePosString(pos, string)
    string.chars.to_a.reduce(pos) do |acc, el|
      updatePosChar acc, el
    end
  end

  def updatePosChar(source_pos, c)
    case c
    when "\n"
      SourcePos(source_pos[0], source_pos[1] + 1, 1)
    when "\t"
      SourcePos(source_pos[0], source_pos[1],
        source_pos[2] + 8 - ((source_pos[2] - 1) % 8))
    else
      SourcePos(source_pos[0], source_pos[1], source_pos[2] + 1)
    end
  end
end
